// created by jay 0.7 (c) 1998 Axel.Schreiner@informatik.uni-osnabrueck.de

#line 2 "parser.jay"

/*
 * parser.jay: parser for Sather
 *
 * Copyright (C) 2003-2004 Shugo Maeda
 * Licensed under the terms of the GNU GPL
 */

using System;
using System.Collections;
using System.IO;
using System.Text;

using Babel.Base;

namespace Babel.Compiler {
    public class Parser {
        protected Program program;
        protected LexicalAnalyzer lexer;
        protected Report report;
        protected SourceFile sourceFile;
        protected ClassDefinition currentClass;

        public Parser(Program program,
                      TextReader reader, string fileName, Report report)
        {
            this.program = program;
            lexer = new LexicalAnalyzer(reader, fileName, report);
            this.report = report;
        }
#line default

  /** simplified error message.
      @see <a href="#yyerror(java.lang.String, java.lang.String[])">yyerror</a>
    */
  public void yyerror (string message) {
    yyerror(message, null);
  }

  /** (syntax) error message.
      Can be overwritten to control message format.
      @param message text to be displayed.
      @param expected vector of acceptable tokens, if available.
    */
  public void yyerror (string message, string[] expected) {
    if ((expected != null) && (expected.Length  > 0)) {
      System.Console.Write (message+", expecting");
      for (int n = 0; n < expected.Length; ++ n)
        System.Console.Write (" "+expected[n]);
        System.Console.WriteLine ();
    } else
      System.Console.WriteLine (message);
  }

  /** debugging support, requires the package jay.yydebug.
      Set to null to suppress debugging messages.
    */
  protected yydebug.yyDebug debug;

  protected static  int yyFinal = 1;
  public static  string [] yyRule = {
    "$accept : source_file",
    "$$1 :",
    "source_file : $$1 class_def_list opt_EOF",
    "opt_EOF :",
    "opt_EOF : EOF",
    "class_def_list : class_def",
    "class_def_list : class_def_list SEMI class_def",
    "class_def :",
    "class_def : abstract_class_definition",
    "class_def : class_definition",
    "$$2 :",
    "abstract_class_definition : ABSTRACT CLASS ABSTRACT_CLASS_NAME subtyping_clause supertyping_clause $$2 IS abstract_signature_list END",
    "subtyping_clause :",
    "subtyping_clause : IS_LT type_specifier_list",
    "type_specifier_list : type_specifier",
    "type_specifier_list : type_specifier_list COMMA type_specifier",
    "type_specifier : class_name",
    "type_specifier : abstract_class_name",
    "type_specifier : SAME",
    "supertyping_clause :",
    "supertyping_clause : IS_GT type_specifier_list",
    "class_name : qualified_identifier",
    "abstract_class_name : ABSTRACT_CLASS_NAME",
    "abstract_class_name : qualified_identifier DCOLON ABSTRACT_CLASS_NAME",
    "qualified_identifier : IDENTIFIER",
    "qualified_identifier : qualified_identifier DCOLON IDENTIFIER",
    "abstract_signature_list : abstract_signature",
    "abstract_signature_list : abstract_signature_list SEMI abstract_signature",
    "abstract_signature :",
    "abstract_signature : abstract_routine_signature",
    "abstract_signature : abstract_iter_signature",
    "abstract_routine_signature : IDENTIFIER opt_routine_argument_list opt_return_type",
    "opt_routine_argument_list :",
    "opt_routine_argument_list : LPAREN routine_argument_list RPAREN",
    "routine_argument_list : routine_argument",
    "routine_argument_list : routine_argument_list COMMA routine_argument",
    "routine_argument : rout_mode_ident_list COLON type_specifier",
    "rout_mode_ident_list : rout_mode_ident",
    "rout_mode_ident_list : rout_mode_ident_list COMMA rout_mode_ident",
    "rout_mode_ident : opt_routine_mode IDENTIFIER",
    "opt_routine_mode :",
    "opt_routine_mode : OUT",
    "opt_routine_mode : INOUT",
    "identifier_list : IDENTIFIER",
    "identifier_list : identifier_list COMMA IDENTIFIER",
    "opt_return_type :",
    "opt_return_type : COLON type_specifier",
    "abstract_iter_signature : iter_name opt_iter_argument_list opt_return_type",
    "iter_name : IDENTIFIER ITER_BANG",
    "opt_iter_argument_list :",
    "opt_iter_argument_list : LPAREN iter_argument_list RPAREN",
    "iter_argument_list : iter_argument",
    "iter_argument_list : iter_argument_list COMMA iter_argument",
    "iter_argument : iter_mode_ident_list COLON type_specifier",
    "iter_mode_ident_list : iter_mode_ident",
    "iter_mode_ident_list : iter_mode_ident_list COMMA iter_mode_ident",
    "iter_mode_ident : opt_iter_mode IDENTIFIER",
    "opt_iter_mode :",
    "opt_iter_mode : OUT",
    "opt_iter_mode : INOUT",
    "opt_iter_mode : ONCE",
    "$$3 :",
    "class_definition : CLASS IDENTIFIER subtyping_clause $$3 IS class_element_list END",
    "class_element_list : class_element",
    "class_element_list : class_element_list SEMI class_element",
    "class_element :",
    "class_element : const_definition",
    "class_element : shared_attr_definition",
    "class_element : attr_definition",
    "class_element : routine_definition",
    "class_element : iter_definition",
    "class_element : include_clause",
    "const_definition : opt_const_modifier CONST IDENTIFIER COLON type_specifier const_initializer",
    "const_definition : opt_const_modifier CONST IDENTIFIER opt_int_const_initializer opt_successive_constant_list",
    "opt_const_modifier :",
    "opt_const_modifier : PRIVATE",
    "const_initializer : ASSIGN literal_expression",
    "opt_int_const_initializer :",
    "opt_int_const_initializer : ASSIGN int_literal_expression",
    "opt_successive_constant_list :",
    "opt_successive_constant_list : COMMA identifier_list",
    "shared_attr_definition : opt_attr_modifier SHARED ATTR identifier_list COLON type_specifier opt_shared_attr_initializer",
    "opt_shared_attr_initializer :",
    "opt_shared_attr_initializer : ASSIGN expression",
    "attr_definition : opt_attr_modifier ATTR identifier_list COLON type_specifier",
    "opt_attr_modifier :",
    "opt_attr_modifier : PRIVATE",
    "opt_attr_modifier : READONLY",
    "routine_definition : opt_routine_modifier IDENTIFIER opt_routine_argument_list opt_return_type opt_pre opt_post IS statement_list END",
    "opt_routine_modifier :",
    "opt_routine_modifier : PRIVATE",
    "opt_pre :",
    "opt_pre : PRE expression",
    "opt_post :",
    "opt_post : POST expression",
    "iter_definition : opt_routine_modifier iter_name opt_iter_argument_list opt_return_type opt_pre opt_post IS statement_list END",
    "include_clause : opt_include_modifier INCLUDE type_specifier opt_feature_modifier_list",
    "opt_include_modifier :",
    "opt_include_modifier : PRIVATE",
    "opt_feature_modifier_list :",
    "opt_feature_modifier_list : feature_modifier_list",
    "feature_modifier_list : feature_modifier",
    "feature_modifier_list : feature_modifier_list COMMA feature_modifier",
    "feature_modifier : IDENTIFIER TRANSFORM opt_new_modifier IDENTIFIER",
    "feature_modifier : IDENTIFIER TRANSFORM",
    "opt_new_modifier :",
    "opt_new_modifier : PRIVATE",
    "opt_new_modifier : READONLY",
    "statement_list : statement",
    "statement_list : statement_list SEMI statement",
    "statement : empty_statement",
    "statement : declaration_statement",
    "statement : assign_statement",
    "statement : if_statement",
    "statement : return_statement",
    "statement : case_statement",
    "statement : typecase_statement",
    "statement : loop_statement",
    "statement : yield_statement",
    "statement : quit_statement",
    "statement : protect_statement",
    "statement : raise_statement",
    "statement : expression_statement",
    "empty_statement :",
    "declaration_statement : identifier_list COLON opt_type_specifier opt_assign",
    "opt_type_specifier :",
    "opt_type_specifier : type_specifier",
    "opt_assign :",
    "opt_assign : ASSIGN expression",
    "assign_statement : IDENTIFIER ASSIGN expression",
    "assign_statement : primary_expression DOT IDENTIFIER ASSIGN expression",
    "assign_statement : type_call ASSIGN expression",
    "assign_statement : primary_expression LBRACKET expr_list RBRACKET ASSIGN expression",
    "if_statement : IF expression THEN statement_list opt_elsif_or_else END",
    "opt_elsif_or_else : opt_else",
    "opt_elsif_or_else : ELSIF expression THEN statement_list opt_elsif_or_else",
    "opt_else :",
    "opt_else : ELSE statement_list",
    "return_statement : RETURN opt_expression",
    "opt_expression :",
    "opt_expression : expression",
    "case_statement : CASE expression case_when_list opt_else END",
    "case_when_list : case_when",
    "case_when_list : case_when_list case_when",
    "case_when : WHEN expression_list THEN statement_list",
    "typecase_statement : TYPECASE local_expression typecase_when_list opt_else END",
    "typecase_when_list : typecase_when",
    "typecase_when_list : typecase_when_list typecase_when",
    "typecase_when : WHEN type_specifier THEN statement_list",
    "expression_list : expression",
    "expression_list : expression_list COMMA expression",
    "loop_statement : LOOP statement_list END",
    "yield_statement : YIELD opt_expression",
    "quit_statement : QUIT",
    "protect_statement : PROTECT statement_list opt_protect_when_list opt_else END",
    "raise_statement : RAISE expression",
    "opt_protect_when_list :",
    "opt_protect_when_list : protect_when_list",
    "protect_when_list : protect_when",
    "protect_when_list : protect_when_list protect_when",
    "protect_when : WHEN type_specifier THEN statement_list",
    "expression_statement : expression",
    "expression : conditional_expression",
    "conditional_expression : relational_expression",
    "conditional_expression : conditional_expression AND relational_expression",
    "conditional_expression : conditional_expression OR relational_expression",
    "relational_expression : additive_expression",
    "relational_expression : relational_expression IS_EQ additive_expression",
    "relational_expression : relational_expression IS_NEQ additive_expression",
    "relational_expression : relational_expression IS_LT additive_expression",
    "relational_expression : relational_expression IS_LEQ additive_expression",
    "relational_expression : relational_expression IS_GT additive_expression",
    "relational_expression : relational_expression IS_GEQ additive_expression",
    "additive_expression : multiplicative_expression",
    "additive_expression : additive_expression PLUS multiplicative_expression",
    "additive_expression : additive_expression MINUS multiplicative_expression",
    "multiplicative_expression : unary_expression",
    "multiplicative_expression : multiplicative_expression TIMES unary_expression",
    "multiplicative_expression : multiplicative_expression QUOTIENT unary_expression",
    "multiplicative_expression : multiplicative_expression MOD unary_expression",
    "unary_expression : pow_expression",
    "$$4 :",
    "unary_expression : MINUS $$4 pow_expression",
    "unary_expression : NOT pow_expression",
    "pow_expression : primary_expression",
    "pow_expression : pow_expression POW primary_expression",
    "primary_expression : literal_expression",
    "primary_expression : self_expression",
    "primary_expression : local_expression",
    "primary_expression : call_expression",
    "primary_expression : type_call_expression",
    "primary_expression : iter_call_expression",
    "primary_expression : void_expression",
    "primary_expression : void_test_expression",
    "primary_expression : new_expression",
    "primary_expression : create_expression",
    "primary_expression : break_expression",
    "primary_expression : while_expression",
    "primary_expression : until_expression",
    "primary_expression : exception_expression",
    "primary_expression : parenthesized_expression",
    "literal_expression : bool_literal_expression",
    "literal_expression : int_literal_expression",
    "literal_expression : char_literal_expression",
    "literal_expression : str_literal_expression",
    "bool_literal_expression : TRUE",
    "bool_literal_expression : FALSE",
    "int_literal_expression : INT_LITERAL",
    "char_literal_expression : CHAR_LITERAL",
    "str_literal_expression : STR_LITERAL",
    "str_literal_expression : str_literal_expression STR_LITERAL",
    "self_expression : SELF",
    "local_expression : IDENTIFIER",
    "call_expression : primary_expression DOT IDENTIFIER opt_mode_expr_list",
    "call_expression : IDENTIFIER LPAREN mode_expr_list RPAREN",
    "call_expression : primary_expression LBRACKET expr_list RBRACKET",
    "type_call_expression : type_call opt_mode_expr_list",
    "iter_call_expression : primary_expression DOT iter_name opt_mode_expr_list",
    "iter_call_expression : iter_name opt_mode_expr_list",
    "opt_mode_expr_list :",
    "opt_mode_expr_list : LPAREN mode_expr_list RPAREN",
    "mode_expr_list : mode_expr",
    "mode_expr_list : mode_expr_list COMMA mode_expr",
    "mode_expr : opt_routine_mode expression",
    "expr_list : expression",
    "expr_list : expr_list COMMA expression",
    "type_call : IDENTIFIER DCOLON IDENTIFIER",
    "type_call : type_call DCOLON IDENTIFIER",
    "void_expression : VOID",
    "void_test_expression : VOID LPAREN expression RPAREN",
    "new_expression : NEW",
    "new_expression : NEW class_name opt_mode_expr_list",
    "create_expression : SHARP type_specifier opt_mode_expr_list",
    "break_expression : BREAK_BANG",
    "while_expression : WHILE_BANG LPAREN expression RPAREN",
    "until_expression : UNTIL_BANG LPAREN expression RPAREN",
    "exception_expression : EXCEPTION",
    "parenthesized_expression : LPAREN expression RPAREN",
  };
  protected static  string [] yyNames = {    
    "end-of-file",null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,"'!'",null,"'#'",null,"'%'",null,
    null,"'('","')'","'*'","'+'","','","'-'","'.'","'/'",null,null,null,
    null,null,null,null,null,null,null,"':'","';'","'<'","'='","'>'",null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,
    "'['",null,"']'","'^'","'_'",null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,"'{'","'|'","'}'","'~'",null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    "EOF","NONE","ERROR","ABSTRACT","AND","ANY","ASSERT","ATTR","BIND",
    "BREAK_BANG","BUILTIN","CASE","CLASS","CLUSTERS","CLUSTERS_BANG",
    "COHORT","CONST","ELSE","ELSIF","END","EXCEPTION","EXTERNAL","FALSE",
    "FAR","FORK","GUARD","IF","IMMUTABLE","INOUT","INCLUDE","INITIAL",
    "IS","ITER","LOCK","LOOP","NEAR","NEW","ONCE","OR","OUT","PAR",
    "PARLOOP","POST","PRE","PRIVATE","PROTECT","QUIT","RAISE","READONLY",
    "RESULT","RETURN","ROUT","F_ROUT","SAME","SELF","SHARED","SYNC",
    "THEN","TRUE","TYPECASE","UNLOCK","UNTIL_BANG","VOID","WHEN",
    "WHILE_BANG","WITH","YIELD","LPAREN","RPAREN","LBRACKET","RBRACKET",
    "LBRACE","RBRACE","COMMA","DOT","SEMI","COLON","UNDER","PLUS","MINUS",
    "TIMES","QUOTIENT","IS_LT","IS_GT","SHARP","BANG","ITER_BANG","POW",
    "MOD","VBAR","IS_NEQ","\"/=\"","IS_LEQ","\"<=\"","IS_GEQ","\">=\"",
    "ASSIGN","\":=\"","TRANSFORM","\"->\"","IS_EQ","NOT","DCOLON",
    "\"::\"","CHAR_LITERAL","STR_LITERAL","INT_LITERAL","FLOAT_LITERAL",
    "IDENTIFIER","ABSTRACT_CLASS_NAME",
  };

  /** index-checked interface to yyNames[].
      @param token single character or %token value.
      @return token name or [illegal] or [unknown].
    */
  public static string yyname (int token) {
    if ((token < 0) || (token > yyNames.Length)) return "[illegal]";
    string name;
    if ((name = yyNames[token]) != null) return name;
    return "[unknown]";
  }

  /** computes list of expected tokens on error by tracing the tables.
      @param state for which to compute the list.
      @return list of token names.
    */
  protected string[] yyExpecting (int state) {
    int token, n, len = 0;
    bool[] ok = new bool[yyNames.Length];

    if ((n = yySindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyNames.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyNames[token] != null) {
          ++ len;
          ok[token] = true;
        }
    if ((n = yyRindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyNames.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyNames[token] != null) {
          ++ len;
          ok[token] = true;
        }

    string [] result = new string[len];
    for (n = token = 0; n < len;  ++ token)
      if (ok[token]) result[n++] = yyNames[token];
    return result;
  }

  /** the generated parser, with debugging messages.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @param yydebug debug message writer implementing yyDebug, or null.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  public Object yyparse (yyParser.yyInput yyLex, Object yyd)
				 {
    this.debug = (yydebug.yyDebug)yyd;
    return yyparse(yyLex);
  }

  /** initial size and increment of the state/value stack [default 256].
      This is not final so that it can be overwritten outside of invocations
      of yyparse().
    */
  protected int yyMax;

  /** executed at the beginning of a reduce action.
      Used as $$ = yyDefault($1), prior to the user-specified action, if any.
      Can be overwritten to provide deep copy, etc.
      @param first value for $1, or null.
      @return first.
    */
  protected Object yyDefault (Object first) {
    return first;
  }

  /** the generated parser.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  public Object yyparse (yyParser.yyInput yyLex)
				{
    if (yyMax <= 0) yyMax = 256;			// initial size
    int yyState = 0;                                   // state stack ptr
    int [] yyStates = new int[yyMax];	                // state stack 
    Object yyVal = null;                               // value stack ptr
    Object [] yyVals = new Object[yyMax];	        // value stack
    int yyToken = -1;					// current input
    int yyErrorFlag = 0;				// #tks to shift

    int yyTop = 0;
    goto skip;
    yyLoop:
    yyTop++;
    skip:
    for (;; ++ yyTop) {
      if (yyTop >= yyStates.Length) {			// dynamically increase
        int[] i = new int[yyStates.Length+yyMax];
        yyStates.CopyTo (i, 0);
        yyStates = i;
        Object[] o = new Object[yyVals.Length+yyMax];
        yyVals.CopyTo (o, 0);
        yyVals = o;
      }
      yyStates[yyTop] = yyState;
      yyVals[yyTop] = yyVal;
      if (debug != null) debug.push(yyState, yyVal);

      yyDiscarded: for (;;) {	// discarding a token does not change stack
        int yyN;
        if ((yyN = yyDefRed[yyState]) == 0) {	// else [default] reduce (yyN)
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
            if (debug != null)
              debug.lex(yyState, yyToken, yyname(yyToken), yyLex.value());
          }
          if ((yyN = yySindex[yyState]) != 0 && ((yyN += yyToken) >= 0)
              && (yyN < yyTable.Length) && (yyCheck[yyN] == yyToken)) {
            if (debug != null)
              debug.shift(yyState, yyTable[yyN], yyErrorFlag-1);
            yyState = yyTable[yyN];		// shift to yyN
            yyVal = yyLex.value();
            yyToken = -1;
            if (yyErrorFlag > 0) -- yyErrorFlag;
            goto yyLoop;
          }
          if ((yyN = yyRindex[yyState]) != 0 && (yyN += yyToken) >= 0
              && yyN < yyTable.Length && yyCheck[yyN] == yyToken)
            yyN = yyTable[yyN];			// reduce (yyN)
          else
            switch (yyErrorFlag) {
  
            case 0:
              yyerror("syntax error", yyExpecting(yyState));
              if (debug != null) debug.error("syntax error");
              goto case 1;
            case 1: case 2:
              yyErrorFlag = 3;
              do {
                if ((yyN = yySindex[yyStates[yyTop]]) != 0
                    && (yyN += Token.yyErrorCode) >= 0 && yyN < yyTable.Length
                    && yyCheck[yyN] == Token.yyErrorCode) {
                  if (debug != null)
                    debug.shift(yyStates[yyTop], yyTable[yyN], 3);
                  yyState = yyTable[yyN];
                  yyVal = yyLex.value();
                  goto yyLoop;
                }
                if (debug != null) debug.pop(yyStates[yyTop]);
              } while (-- yyTop >= 0);
              if (debug != null) debug.reject();
              throw new yyParser.yyException("irrecoverable syntax error");
  
            case 3:
              if (yyToken == 0) {
                if (debug != null) debug.reject();
                throw new yyParser.yyException("irrecoverable syntax error at end-of-file");
              }
              if (debug != null)
                debug.discard(yyState, yyToken, yyname(yyToken),
  							yyLex.value());
              yyToken = -1;
              goto yyDiscarded;		// leave stack alone
            }
        }
        int yyV = yyTop + 1-yyLen[yyN];
        if (debug != null)
          debug.reduce(yyState, yyStates[yyV-1], yyN, yyRule[yyN], yyLen[yyN]);
        yyVal = yyDefault(yyV > yyTop ? null : yyVals[yyV]);
        switch (yyN) {
case 1:
#line 150 "parser.jay"
  {
              sourceFile = new SourceFile();
          }
  break;
case 2:
#line 155 "parser.jay"
  {
              program.AddChild(sourceFile);
          }
  break;
case 10:
#line 182 "parser.jay"
  {
              currentClass = new ClassDefinition((string) yyVals[-2+yyTop],
                                                 ClassKind.Abstract,
                                                 (TypedNodeList) yyVals[-1+yyTop],
                                                 (TypedNodeList) yyVals[0+yyTop],
                                                 lexer.Location);
	  }
  break;
case 11:
#line 192 "parser.jay"
  {
              AddClass(currentClass);
              yyVal = currentClass;
	  }
  break;
case 12:
#line 200 "parser.jay"
  {
              yyVal = new TypedNodeList();
          }
  break;
case 13:
#line 204 "parser.jay"
  {
              yyVal = new TypedNodeList((Node) yyVals[0+yyTop]);
          }
  break;
case 15:
#line 212 "parser.jay"
  {
              Node list = (Node) yyVals[-2+yyTop];
              list.Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 16:
#line 221 "parser.jay"
  {
              yyVal = new TypeSpecifier((string) yyVals[0+yyTop], TypeKind.Normal,
                                     lexer.Location);
          }
  break;
case 17:
#line 226 "parser.jay"
  {
              yyVal = new TypeSpecifier((string) yyVals[0+yyTop], TypeKind.Normal,
                                     lexer.Location);
          }
  break;
case 18:
#line 231 "parser.jay"
  {
              yyVal = new TypeSpecifier((string) yyVals[0+yyTop], TypeKind.Same,
                                     lexer.Location);
          }
  break;
case 19:
#line 239 "parser.jay"
  {
              yyVal = new TypedNodeList();
          }
  break;
case 20:
#line 243 "parser.jay"
  {
              yyVal = new TypedNodeList((Node) yyVals[0+yyTop]);
          }
  break;
case 23:
#line 255 "parser.jay"
  {
              yyVal = (string) yyVals[-2+yyTop] + Type.Delimiter + (string) yyVals[0+yyTop];
          }
  break;
case 25:
#line 263 "parser.jay"
  {
              yyVal = (string) yyVals[-2+yyTop] + Type.Delimiter + (string) yyVals[0+yyTop];
          }
  break;
case 31:
#line 283 "parser.jay"
  {
              AbstractRoutineSignature routine =
                  new AbstractRoutineSignature((string) yyVals[-2+yyTop],
                                               (TypedNodeList) yyVals[-1+yyTop],
                                               (TypeSpecifier) yyVals[0+yyTop],
                                               lexer.Location);
              currentClass.AddChild(routine);
          }
  break;
case 32:
#line 295 "parser.jay"
  {
              yyVal = new TypedNodeList();
          }
  break;
case 33:
#line 299 "parser.jay"
  {
              yyVal = new TypedNodeList((Node) yyVals[-1+yyTop]);
          }
  break;
case 35:
#line 307 "parser.jay"
  {
              Node args = (Node) yyVals[-2+yyTop];
              args.Append((Node) yyVals[0+yyTop]);
              yyVal = args;
          }
  break;
case 36:
#line 316 "parser.jay"
  {
              Argument args = (Argument) yyVals[-2+yyTop];
              foreach (Argument arg in args) {
                  arg.TypeSpecifier = (TypeSpecifier) yyVals[0+yyTop];
              }
              yyVal = args;
          }
  break;
case 38:
#line 328 "parser.jay"
  {
              ((Node) yyVals[-2+yyTop]).Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 39:
#line 336 "parser.jay"
  {
              yyVal = new Argument((ArgumentMode) yyVals[-1+yyTop], (string) yyVals[0+yyTop], null,
                                lexer.Location);
          }
  break;
case 40:
#line 344 "parser.jay"
  {
              yyVal = ArgumentMode.In;
          }
  break;
case 41:
#line 348 "parser.jay"
  {
              yyVal = ArgumentMode.Out;
          }
  break;
case 42:
#line 352 "parser.jay"
  {
              yyVal = ArgumentMode.InOut;
          }
  break;
case 43:
#line 359 "parser.jay"
  {
              yyVal = NewList(yyVals[0+yyTop]);
          }
  break;
case 44:
#line 363 "parser.jay"
  {
              yyVal = ListAppend(yyVals[-2+yyTop], yyVals[0+yyTop]);
          }
  break;
case 45:
#line 370 "parser.jay"
  {
              yyVal = TypeSpecifier.Null;
          }
  break;
case 46:
#line 374 "parser.jay"
  {
              yyVal = yyVals[0+yyTop];
          }
  break;
case 47:
#line 383 "parser.jay"
  {
              AbstractIterSignature iter =
                  new AbstractIterSignature((string) yyVals[-2+yyTop],
                                            (TypedNodeList) yyVals[-1+yyTop],
                                            (TypeSpecifier) yyVals[0+yyTop],
                                            lexer.Location);
              currentClass.AddChild(iter);
          }
  break;
case 48:
#line 395 "parser.jay"
  {
              yyVal = (string) yyVals[-1+yyTop] + "!";
          }
  break;
case 49:
#line 402 "parser.jay"
  {
              yyVal = new TypedNodeList();
          }
  break;
case 50:
#line 406 "parser.jay"
  {
              yyVal = new TypedNodeList((Node) yyVals[-1+yyTop]);
          }
  break;
case 52:
#line 414 "parser.jay"
  {
              Node args = (Node) yyVals[-2+yyTop];
              args.Append((Node) yyVals[0+yyTop]);
              yyVal = args;
          }
  break;
case 53:
#line 423 "parser.jay"
  {
              Argument args = (Argument) yyVals[-2+yyTop];
              foreach (Argument arg in args) {
                  arg.TypeSpecifier = (TypeSpecifier) yyVals[0+yyTop];
              }
              yyVal = args;
          }
  break;
case 55:
#line 435 "parser.jay"
  {
              ((Node) yyVals[-2+yyTop]).Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 56:
#line 443 "parser.jay"
  {
              yyVal = new Argument((ArgumentMode) yyVals[-1+yyTop], (string) yyVals[0+yyTop], null,
                                lexer.Location);
          }
  break;
case 57:
#line 451 "parser.jay"
  {
              yyVal = ArgumentMode.In;
          }
  break;
case 58:
#line 455 "parser.jay"
  {
              yyVal = ArgumentMode.Out;
          }
  break;
case 59:
#line 459 "parser.jay"
  {
              yyVal = ArgumentMode.InOut;
          }
  break;
case 60:
#line 463 "parser.jay"
  {
              yyVal = ArgumentMode.Once;
          }
  break;
case 61:
#line 472 "parser.jay"
  {
              currentClass = new ClassDefinition((string) yyVals[-1+yyTop],
                                                 ClassKind.Reference,
                                                 (TypedNodeList) yyVals[0+yyTop],
                                                 lexer.Location);
	  }
  break;
case 62:
#line 481 "parser.jay"
  {
              AddClass(currentClass);
              yyVal = currentClass;
	  }
  break;
case 72:
#line 505 "parser.jay"
  {
              ConstDefinition constDef =
                  new ConstDefinition((string) yyVals[-3+yyTop], (TypeSpecifier) yyVals[-1+yyTop],
                                      ((LiteralExpression) yyVals[0+yyTop]).ValueAsObject,
                                      (ConstModifier) yyVals[-5+yyTop], lexer.Location);
              currentClass.AddChild(constDef);
          }
  break;
case 73:
#line 514 "parser.jay"
  {
              int value = (int) yyVals[-1+yyTop];
              TypeSpecifier type = new TypeSpecifier("INT", TypeKind.Normal,
                                                     lexer.Location);
              ConstDefinition constDef =
                  new ConstDefinition((string) yyVals[-2+yyTop], type, value,
                                      (ConstModifier) yyVals[-4+yyTop], lexer.Location);
              currentClass.AddChild(constDef);
              if (yyVals[0+yyTop] != null) {
                  ArrayList identifierList = (ArrayList) yyVals[0+yyTop];
                  foreach (string identifier in identifierList) {
                      constDef =
                          new ConstDefinition(identifier, type, ++value,
                                              (ConstModifier) yyVals[-4+yyTop],
                                              lexer.Location);
                      currentClass.AddChild(constDef);
                  }
              }
          }
  break;
case 74:
#line 537 "parser.jay"
  {
              yyVal = ConstModifier.None;
          }
  break;
case 75:
#line 541 "parser.jay"
  {
              yyVal = ConstModifier.Private;
          }
  break;
case 76:
#line 548 "parser.jay"
  {
              yyVal = yyVals[0+yyTop];
          }
  break;
case 77:
#line 555 "parser.jay"
  {
              yyVal = 0;
          }
  break;
case 78:
#line 559 "parser.jay"
  {
              yyVal = ((IntLiteralExpression) yyVals[0+yyTop]).Value;
          }
  break;
case 80:
#line 567 "parser.jay"
  {
              yyVal = yyVals[0+yyTop];
          }
  break;
case 81:
#line 575 "parser.jay"
  {
              ArrayList identifierList = (ArrayList) yyVals[-3+yyTop];
              if (yyVals[0+yyTop] != null && identifierList.Count > 1) {
                  report.Error(lexer.Location,
                               "can't initialize multiple shared attrs");
              }
              foreach (string identifier in identifierList) {
                  SharedAttrDefinition attr =
                      new SharedAttrDefinition(identifier,
                                               (TypeSpecifier) yyVals[-1+yyTop],
                                               (Expression) yyVals[0+yyTop],
                                               (AttrModifier) yyVals[-6+yyTop],
                                               lexer.Location);
                  currentClass.AddChild(attr);
              }
          }
  break;
case 83:
#line 596 "parser.jay"
  {
              yyVal = yyVals[0+yyTop];
          }
  break;
case 84:
#line 603 "parser.jay"
  {
              ArrayList identifierList = (ArrayList) yyVals[-2+yyTop];
              foreach (string identifier in identifierList) {
                  AttrDefinition attr = new AttrDefinition(identifier,
                                                           (TypeSpecifier) yyVals[0+yyTop],
                                                           (AttrModifier) yyVals[-4+yyTop],
                                                           lexer.Location);
                  currentClass.AddChild(attr);
              }
          }
  break;
case 85:
#line 617 "parser.jay"
  {
              yyVal = AttrModifier.None;
          }
  break;
case 86:
#line 621 "parser.jay"
  {
              yyVal = AttrModifier.Private;
          }
  break;
case 87:
#line 625 "parser.jay"
  {
              yyVal = AttrModifier.Readonly;
          }
  break;
case 88:
#line 640 "parser.jay"
  {
              RoutineDefinition routine =
                  new RoutineDefinition((string) yyVals[-7+yyTop],
                                        (TypedNodeList) yyVals[-6+yyTop],
                                        (TypeSpecifier) yyVals[-5+yyTop],
                                        (StatementList) yyVals[-1+yyTop],
                                        (RoutineModifier) yyVals[-8+yyTop],
                                        lexer.Location);
              currentClass.AddChild(routine);
          }
  break;
case 89:
#line 654 "parser.jay"
  {
              yyVal = RoutineModifier.None;
          }
  break;
case 90:
#line 658 "parser.jay"
  {
              yyVal = RoutineModifier.Private;
          }
  break;
case 95:
#line 683 "parser.jay"
  {
              IterDefinition iter =
                  new IterDefinition((string) yyVals[-7+yyTop],
                                     (TypedNodeList) yyVals[-6+yyTop],
                                     (TypeSpecifier) yyVals[-5+yyTop],
                                     (StatementList) yyVals[-1+yyTop],
                                     (RoutineModifier) yyVals[-8+yyTop],
                                     lexer.Location);
              currentClass.AddChild(iter);
          }
  break;
case 96:
#line 697 "parser.jay"
  {
              IncludeClause include
                  = new IncludeClause((TypeSpecifier) yyVals[-1+yyTop], (IncludeModifier) yyVals[-3+yyTop],
                                      (NodeList) yyVals[0+yyTop],
                                      lexer.Location);
              currentClass.AddChild(include);
          }
  break;
case 97:
#line 708 "parser.jay"
  {
              yyVal = IncludeModifier.NoChange;
          }
  break;
case 98:
#line 712 "parser.jay"
  {
              yyVal = IncludeModifier.Private;
          }
  break;
case 99:
#line 719 "parser.jay"
  {
              yyVal = new NodeList();
          }
  break;
case 101:
#line 727 "parser.jay"
  {
              yyVal = new NodeList((Node) yyVals[0+yyTop]);
          }
  break;
case 102:
#line 731 "parser.jay"
  {
              ((NodeList) yyVals[-2+yyTop]).Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 103:
#line 739 "parser.jay"
  {
              yyVal = new FeatureModifier((string) yyVals[-3+yyTop],
                                       (string) yyVals[0+yyTop],
                                       (IncludeModifier) yyVals[-1+yyTop],
                                       lexer.Location);
          }
  break;
case 104:
#line 746 "parser.jay"
  {
              yyVal = new FeatureModifier((string) yyVals[-1+yyTop],
                                       (string) "",
                                       IncludeModifier.None,
                                       lexer.Location);
          }
  break;
case 105:
#line 756 "parser.jay"
  {
              yyVal = IncludeModifier.None;
          }
  break;
case 106:
#line 760 "parser.jay"
  {
              yyVal = IncludeModifier.Private;
          }
  break;
case 107:
#line 764 "parser.jay"
  {
              yyVal = IncludeModifier.Readonly;
          }
  break;
case 108:
#line 771 "parser.jay"
  {
              StatementList list = new StatementList(lexer.Location);
              list.AddChild((Node) yyVals[0+yyTop]);
              yyVal = list;
          }
  break;
case 109:
#line 777 "parser.jay"
  {
              ((StatementList) yyVals[-2+yyTop]).AddChild((Statement) yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 123:
#line 801 "parser.jay"
  {
              yyVal = new EmptyStatement(lexer.Location);
          }
  break;
case 124:
#line 808 "parser.jay"
  {
              ArrayList identifierList = (ArrayList) yyVals[-3+yyTop];
              DeclarationStatement decls = null;
              foreach (string identifier in identifierList) {
                  DeclarationStatement decl =
                      new DeclarationStatement(identifier,
                                               (TypeSpecifier) yyVals[-1+yyTop],
                                               lexer.Location);
                  if (decls == null) {
                      decls = decl;
                  }
                  else {
                      decls.Append(decl);
                  }
              }
              if (yyVals[-1+yyTop] == null && yyVals[0+yyTop] == null)
                  throw new yyParser.yyException("type expected");
              if (yyVals[0+yyTop] != null) {
                  if (identifierList.Count > 1)
                      throw new yyParser.yyException("can't initialize multiple variables");
                  AssignStatement assign =
                       new AssignStatement((string) identifierList[0],
                                           (Expression) yyVals[0+yyTop],
                                           lexer.Location);
                  decls.Append(assign);
              }
              yyVal = decls;
          }
  break;
case 125:
#line 840 "parser.jay"
  {
              yyVal = TypeSpecifier.Null;
          }
  break;
case 128:
#line 849 "parser.jay"
  {
              yyVal = yyVals[0+yyTop];
          }
  break;
case 129:
#line 856 "parser.jay"
  {
              yyVal = new AssignStatement((string) yyVals[-2+yyTop], (Expression) yyVals[0+yyTop],
                                       lexer.Location);
          }
  break;
case 130:
#line 861 "parser.jay"
  {
              Expression expr = new CallExpression((Expression) yyVals[-4+yyTop],
                                                   (string) yyVals[-2+yyTop],
                                                   OneInArg((Expression) yyVals[0+yyTop]),
                                                   lexer.Location);
              yyVal = new ExpressionStatement(expr, lexer.Location);
          }
  break;
case 131:
#line 869 "parser.jay"
  {
              ArrayList list = (ArrayList) yyVals[-2+yyTop];
              string routineName = (string) list[list.Count - 1];
              list.RemoveAt(list.Count - 1);
              string typeName = "";
              foreach (string s in list) {
                  if (typeName.Length > 0)
                      typeName += Type.Delimiter;
                  typeName += s;
              }
              TypeSpecifier typeSpecifier =
                  new TypeSpecifier(typeName, TypeKind.Normal,
                                    lexer.Location);
              Expression expr = new CallExpression(typeSpecifier,
                                                   routineName,
                                                   OneInArg((Expression) yyVals[0+yyTop]),
                                                   lexer.Location);
              yyVal = new ExpressionStatement(expr, lexer.Location);
          }
  break;
case 132:
#line 889 "parser.jay"
  {
              TypedNodeList args = (TypedNodeList) yyVals[-3+yyTop];
              args.Append(InArg((Expression) yyVals[0+yyTop]));
              Expression expr = new CallExpression((Expression) yyVals[-5+yyTop],
                                                   "aset", args,
                                                   lexer.Location);
              yyVal = new ExpressionStatement(expr, expr.Location);
          }
  break;
case 133:
#line 902 "parser.jay"
  {
              yyVal = new IfStatement((Expression) yyVals[-4+yyTop], (Node) yyVals[-2+yyTop], (Node) yyVals[-1+yyTop],
                                   lexer.Location);
          }
  break;
case 135:
#line 912 "parser.jay"
  {
              yyVal = new IfStatement((Expression) yyVals[-3+yyTop], (Node) yyVals[-1+yyTop], (Node) yyVals[0+yyTop],
                                   lexer.Location);
          }
  break;
case 137:
#line 921 "parser.jay"
  {
              yyVal = yyVals[0+yyTop];
          }
  break;
case 138:
#line 928 "parser.jay"
  {
              yyVal = new ReturnStatement((Expression) yyVals[0+yyTop], lexer.Location);
          }
  break;
case 141:
#line 940 "parser.jay"
  {
              yyVal = new CaseStatement((Expression) yyVals[-3+yyTop],
                                     (NodeList) yyVals[-2+yyTop],
                                     (StatementList) yyVals[-1+yyTop],
                                     lexer.Location);
          }
  break;
case 142:
#line 950 "parser.jay"
  {
              yyVal = new NodeList((Node) yyVals[0+yyTop]);
          }
  break;
case 143:
#line 954 "parser.jay"
  {
              ((NodeList) yyVals[-1+yyTop]).Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-1+yyTop];
          }
  break;
case 144:
#line 962 "parser.jay"
  {
              yyVal = new CaseWhen((TypedNodeList) yyVals[-2+yyTop], (StatementList) yyVals[0+yyTop],
                                lexer.Location);
          }
  break;
case 145:
#line 970 "parser.jay"
  {
              yyVal = new TypecaseStatement((LocalExpression) yyVals[-3+yyTop],
                                         (NodeList) yyVals[-2+yyTop],
                                         (StatementList) yyVals[-1+yyTop],
                                         lexer.Location);
          }
  break;
case 146:
#line 980 "parser.jay"
  {
              yyVal = new NodeList((Node) yyVals[0+yyTop]);
          }
  break;
case 147:
#line 984 "parser.jay"
  {
              ((NodeList) yyVals[-1+yyTop]).Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-1+yyTop];
          }
  break;
case 148:
#line 992 "parser.jay"
  {
              yyVal = new TypecaseWhen((TypeSpecifier) yyVals[-2+yyTop], (StatementList) yyVals[0+yyTop],
                                    lexer.Location);
          }
  break;
case 149:
#line 1000 "parser.jay"
  {
              yyVal = new TypedNodeList((TypedNode) yyVals[0+yyTop]);
          }
  break;
case 150:
#line 1004 "parser.jay"
  {
              ((TypedNodeList) yyVals[-2+yyTop]).Append((TypedNode) yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 151:
#line 1012 "parser.jay"
  {
              yyVal = new LoopStatement((StatementList) yyVals[-1+yyTop], lexer.Location);
          }
  break;
case 152:
#line 1019 "parser.jay"
  {
              yyVal = new YieldStatement((Expression) yyVals[0+yyTop], lexer.Location);
          }
  break;
case 153:
#line 1026 "parser.jay"
  {
              yyVal = new QuitStatement(lexer.Location);
          }
  break;
case 154:
#line 1033 "parser.jay"
  {
              yyVal = new ProtectStatement((StatementList) yyVals[-3+yyTop],
                                        (NodeList) yyVals[-2+yyTop],
                                        (StatementList) yyVals[-1+yyTop],
                                        lexer.Location);
          }
  break;
case 155:
#line 1043 "parser.jay"
  {
              yyVal = new RaiseStatement((Expression) yyVals[0+yyTop], lexer.Location);
          }
  break;
case 156:
#line 1050 "parser.jay"
  {
              yyVal = new NodeList();
          }
  break;
case 158:
#line 1058 "parser.jay"
  {
              yyVal = new NodeList((Node) yyVals[0+yyTop]);
          }
  break;
case 159:
#line 1062 "parser.jay"
  {
              ((NodeList) yyVals[-1+yyTop]).Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-1+yyTop];
          }
  break;
case 160:
#line 1070 "parser.jay"
  {
              yyVal = new ProtectWhen((TypeSpecifier) yyVals[-2+yyTop], (StatementList) yyVals[0+yyTop],
                                   lexer.Location);
          }
  break;
case 161:
#line 1078 "parser.jay"
  {
              Expression expr = (Expression) yyVals[0+yyTop];
              yyVal = new ExpressionStatement(expr, expr.Location);
          }
  break;
case 164:
#line 1091 "parser.jay"
  {
              yyVal = new AndExpression((Expression) yyVals[-2+yyTop], (Expression) yyVals[0+yyTop],
                                     lexer.Location);
          }
  break;
case 165:
#line 1096 "parser.jay"
  {
              yyVal = new OrExpression((Expression) yyVals[-2+yyTop], (Expression) yyVals[0+yyTop],
                                    lexer.Location);
          }
  break;
case 167:
#line 1105 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "is_eq",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 168:
#line 1111 "parser.jay"
  {
              Expression callIsEq =
                  new CallExpression((Expression) yyVals[-2+yyTop], "is_eq",
                                     OneInArg((Expression) yyVals[0+yyTop]),
                                     lexer.Location);
              yyVal = new CallExpression(callIsEq, "not", new TypedNodeList(),
                                      lexer.Location);
          }
  break;
case 169:
#line 1120 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "is_lt",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 170:
#line 1126 "parser.jay"
  {
              Expression callIsLt =
                  new CallExpression((Expression) yyVals[0+yyTop], "is_lt",
                                     OneInArg((Expression) yyVals[-2+yyTop]), true,
                                     lexer.Location);
              yyVal = new CallExpression(callIsLt, "not", new TypedNodeList(),
                                      lexer.Location);
          }
  break;
case 171:
#line 1135 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[0+yyTop], "is_lt",
                                      OneInArg((Expression) yyVals[-2+yyTop]), true,
                                      lexer.Location);
          }
  break;
case 172:
#line 1141 "parser.jay"
  {
              Expression callIsLt =
                  new CallExpression((Expression) yyVals[-2+yyTop], "is_lt",
                                     OneInArg((Expression) yyVals[0+yyTop]),
                                     lexer.Location);
              yyVal = new CallExpression(callIsLt, "not", new TypedNodeList(),
                                      lexer.Location);
          }
  break;
case 174:
#line 1154 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "plus",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 175:
#line 1160 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "minus",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 177:
#line 1170 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "times",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 178:
#line 1176 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "quotient",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 179:
#line 1182 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "mod",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 181:
#line 1192 "parser.jay"
  {
              yyVal = ((LexicalAnalyzer) yyLex).lookahead() == Token.INT_LITERAL;
          }
  break;
case 182:
#line 1196 "parser.jay"
  {
              Expression expr = (Expression) yyVals[0+yyTop];
              if (expr is IntLiteralExpression) {
                  IntLiteralExpression intLiteral =
                      (IntLiteralExpression) expr;
                  yyVal = new IntLiteralExpression(-intLiteral.Value,
                                                lexer.Location);
              }
              else {
                  bool followedByLiteral = (bool) yyVals[-1+yyTop];
                  if (followedByLiteral) {
                      report.Error(lexer.Location,
                                   "use parens to disambiguate unary minus " +
                                   "from negative literals");
                  }
                  yyVal = new CallExpression(expr, "negate",
                                          new TypedNodeList(),
                                          lexer.Location);
              }
          }
  break;
case 183:
#line 1217 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[0+yyTop], "not",
                                      new TypedNodeList(),
                                      lexer.Location);
          }
  break;
case 185:
#line 1227 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-2+yyTop], "pow",
                                      OneInArg((Expression) yyVals[0+yyTop]),
                                      lexer.Location);
          }
  break;
case 205:
#line 1261 "parser.jay"
  {
              yyVal = new BoolLiteralExpression(true, lexer.Location);
          }
  break;
case 206:
#line 1265 "parser.jay"
  {
              yyVal = new BoolLiteralExpression(false, lexer.Location);
          }
  break;
case 207:
#line 1272 "parser.jay"
  {
              yyVal = new IntLiteralExpression((int) yyVals[0+yyTop], lexer.Location);
          }
  break;
case 208:
#line 1279 "parser.jay"
  {
              yyVal = new CharLiteralExpression((char) yyVals[0+yyTop], lexer.Location);
          }
  break;
case 209:
#line 1286 "parser.jay"
  {
              yyVal = new StrLiteralExpression((string) yyVals[0+yyTop], lexer.Location);
          }
  break;
case 210:
#line 1290 "parser.jay"
  {
              StrLiteralExpression expr = (StrLiteralExpression) yyVals[-1+yyTop];
              expr.Value += (string) yyVals[0+yyTop];
              expr.Location = lexer.Location;
              yyVal = expr;
          }
  break;
case 211:
#line 1300 "parser.jay"
  {
              yyVal = new SelfExpression(lexer.Location);
          }
  break;
case 212:
#line 1307 "parser.jay"
  {
              yyVal = new LocalExpression((string) yyVals[0+yyTop], lexer.Location);
          }
  break;
case 213:
#line 1314 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-3+yyTop], (string) yyVals[-1+yyTop],
                                      (TypedNodeList) yyVals[0+yyTop],
                                      lexer.Location);
          }
  break;
case 214:
#line 1320 "parser.jay"
  {
              Expression self = new SelfExpression(lexer.Location);
              yyVal = new CallExpression(self, (string) yyVals[-3+yyTop], (TypedNodeList) yyVals[-1+yyTop],
                                      lexer.Location);
          }
  break;
case 215:
#line 1326 "parser.jay"
  {
              yyVal = new CallExpression((Expression) yyVals[-3+yyTop], "aget",
                                      (TypedNodeList) yyVals[-1+yyTop],
                                      lexer.Location);
          }
  break;
case 216:
#line 1335 "parser.jay"
  {
              ArrayList list = (ArrayList) yyVals[-1+yyTop];
              string routineName = (string) list[list.Count - 1];
              list.RemoveAt(list.Count - 1);
              string typeName = "";
              foreach (string s in list) {
                  if (typeName.Length > 0)
                      typeName += Type.Delimiter;
                  typeName += s;
              }
              TypeSpecifier typeSpecifier =
                  new TypeSpecifier(typeName, TypeKind.Normal,
                                    lexer.Location);
              yyVal = new CallExpression(typeSpecifier, routineName,
                                      (TypedNodeList) yyVals[0+yyTop],
                                      lexer.Location);
          }
  break;
case 217:
#line 1356 "parser.jay"
  {
              yyVal = new IterCallExpression((Expression) yyVals[-3+yyTop], (string) yyVals[-1+yyTop],
                                          (TypedNodeList) yyVals[0+yyTop],
                                          lexer.Location);
          }
  break;
case 218:
#line 1362 "parser.jay"
  {
              Expression self = new SelfExpression(lexer.Location);
              yyVal = new IterCallExpression(self, (string) yyVals[-1+yyTop], (TypedNodeList) yyVals[0+yyTop],
                                          lexer.Location);
          }
  break;
case 219:
#line 1371 "parser.jay"
  {
              yyVal = new TypedNodeList();
          }
  break;
case 220:
#line 1375 "parser.jay"
  {
              yyVal = yyVals[-1+yyTop];
          }
  break;
case 221:
#line 1382 "parser.jay"
  {
              yyVal = new TypedNodeList((Node) yyVals[0+yyTop]);
          }
  break;
case 222:
#line 1386 "parser.jay"
  {
              ((TypedNodeList) yyVals[-2+yyTop]).Append((Node) yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 223:
#line 1394 "parser.jay"
  {
              yyVal = new ModalExpression((ArgumentMode) yyVals[-1+yyTop],
                                       (Expression) yyVals[0+yyTop],
                                       lexer.Location);
          }
  break;
case 224:
#line 1403 "parser.jay"
  {
              yyVal = new TypedNodeList(InArg((Expression) yyVals[0+yyTop]));
          }
  break;
case 225:
#line 1407 "parser.jay"
  {
              ((TypedNodeList) yyVals[-2+yyTop]).Append(InArg((Expression) yyVals[0+yyTop]));
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 226:
#line 1415 "parser.jay"
  {
              ArrayList list = new ArrayList();
              list.Add(yyVals[-2+yyTop]);
              list.Add(yyVals[0+yyTop]);
              yyVal = list;
          }
  break;
case 227:
#line 1422 "parser.jay"
  {
              ((ArrayList) yyVals[-2+yyTop]).Add(yyVals[0+yyTop]);
              yyVal = yyVals[-2+yyTop];
          }
  break;
case 228:
#line 1430 "parser.jay"
  {
              yyVal = new VoidExpression(lexer.Location);
          }
  break;
case 229:
#line 1437 "parser.jay"
  {
              yyVal = new VoidTestExpression((Expression) yyVals[-1+yyTop], lexer.Location);
          }
  break;
case 230:
#line 1444 "parser.jay"
  {
              yyVal = new NewExpression(lexer.Location);
          }
  break;
case 231:
#line 1448 "parser.jay"
  {
              TypeSpecifier typeSpecifier =
                  new TypeSpecifier((string) yyVals[-1+yyTop], TypeKind.Normal,
                                    lexer.Location);
              yyVal = new NewExpression(typeSpecifier, (TypedNodeList) yyVals[0+yyTop],
                                     lexer.Location);
          }
  break;
case 232:
#line 1459 "parser.jay"
  {
              yyVal = new CallExpression((TypeSpecifier) yyVals[-1+yyTop], "create",
                                      (TypedNodeList) yyVals[0+yyTop],
                                      lexer.Location);
          }
  break;
case 233:
#line 1468 "parser.jay"
  {
              yyVal = new BreakExpression(lexer.Location);
          }
  break;
case 234:
#line 1475 "parser.jay"
  {
              EmptyStatement empty = new EmptyStatement(lexer.Location);
              BreakExpression breakExpr = new BreakExpression(lexer.Location);
              yyVal = new IfExpression((Expression) yyVals[-1+yyTop], empty, breakExpr,
                                    lexer.Location);
          }
  break;
case 235:
#line 1485 "parser.jay"
  {
              EmptyStatement empty = new EmptyStatement(lexer.Location);
              BreakExpression breakExpr = new BreakExpression(lexer.Location);
              yyVal = new IfExpression((Expression) yyVals[-1+yyTop], breakExpr, empty,
                                    lexer.Location);
          }
  break;
case 236:
#line 1495 "parser.jay"
  {
              yyVal = new ExceptionExpression(lexer.Location);
          }
  break;
case 237:
#line 1502 "parser.jay"
  {
              yyVal = yyVals[-1+yyTop];
          }
  break;
#line default
        }
        yyTop -= yyLen[yyN];
        yyState = yyStates[yyTop];
        int yyM = yyLhs[yyN];
        if (yyState == 0 && yyM == 0) {
          if (debug != null) debug.shift(0, yyFinal);
          yyState = yyFinal;
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
            if (debug != null)
               debug.lex(yyState, yyToken,yyname(yyToken), yyLex.value());
          }
          if (yyToken == 0) {
            if (debug != null) debug.accept(yyVal);
            return yyVal;
          }
          goto yyLoop;
        }
        if (((yyN = yyGindex[yyM]) != 0) && ((yyN += yyState) >= 0)
            && (yyN < yyTable.Length) && (yyCheck[yyN] == yyState))
          yyState = yyTable[yyN];
        else
          yyState = yyDgoto[yyM];
        if (debug != null) debug.shift(yyStates[yyTop], yyState);
	 goto yyLoop;
      }
    }
  }

   static  short [] yyLhs  = {              -1,
    2,    0,    3,    3,    1,    1,    4,    4,    4,    9,
    5,    7,    7,   11,   11,   12,   12,   12,    8,    8,
   13,   14,   14,   15,   15,   10,   10,   16,   16,   16,
   17,   19,   19,   21,   21,   22,   23,   23,   24,   25,
   25,   25,   26,   26,   20,   20,   18,   27,   28,   28,
   29,   29,   30,   31,   31,   32,   33,   33,   33,   33,
   34,    6,   35,   35,   36,   36,   36,   36,   36,   36,
   36,   37,   37,   43,   43,   44,   45,   45,   46,   46,
   38,   50,   50,   39,   49,   49,   49,   40,   52,   52,
   53,   53,   54,   54,   41,   42,   56,   56,   57,   57,
   58,   58,   59,   59,   60,   60,   60,   55,   55,   61,
   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,
   61,   61,   62,   63,   75,   75,   76,   76,   64,   64,
   64,   64,   65,   80,   80,   81,   81,   66,   82,   82,
   67,   83,   83,   84,   68,   87,   87,   88,   85,   85,
   69,   70,   71,   72,   73,   89,   89,   90,   90,   91,
   74,   51,   92,   92,   92,   93,   93,   93,   93,   93,
   93,   93,   94,   94,   94,   95,   95,   95,   95,   96,
   98,   96,   96,   97,   97,   77,   77,   77,   77,   77,
   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,
   47,   47,   47,   47,  112,  112,   48,  113,  114,  114,
   99,   86,  100,  100,  100,  101,  102,  102,  115,  115,
  116,  116,  117,   79,   79,   78,   78,  103,  104,  105,
  105,  106,  107,  108,  109,  110,  111,
  };
   static  short [] yyLen = {           2,
    0,    3,    0,    1,    1,    3,    0,    1,    1,    0,
    9,    0,    2,    1,    3,    1,    1,    1,    0,    2,
    1,    1,    3,    1,    3,    1,    3,    0,    1,    1,
    3,    0,    3,    1,    3,    3,    1,    3,    2,    0,
    1,    1,    1,    3,    0,    2,    3,    2,    0,    3,
    1,    3,    3,    1,    3,    2,    0,    1,    1,    1,
    0,    7,    1,    3,    0,    1,    1,    1,    1,    1,
    1,    6,    5,    0,    1,    2,    0,    2,    0,    2,
    7,    0,    2,    5,    0,    1,    1,    9,    0,    1,
    0,    2,    0,    2,    9,    4,    0,    1,    0,    1,
    1,    3,    4,    2,    0,    1,    1,    1,    3,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    0,    4,    0,    1,    0,    2,    3,    5,
    3,    6,    6,    1,    5,    0,    2,    2,    0,    1,
    5,    1,    2,    4,    5,    1,    2,    4,    1,    3,
    3,    2,    1,    5,    2,    0,    1,    1,    2,    4,
    1,    1,    1,    3,    3,    1,    3,    3,    3,    3,
    3,    3,    1,    3,    3,    1,    3,    3,    3,    1,
    0,    3,    2,    1,    3,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
    1,    1,    4,    4,    4,    2,    4,    2,    0,    3,
    1,    3,    2,    1,    3,    3,    3,    1,    4,    1,
    3,    3,    1,    4,    4,    1,    3,
  };
   static  short [] yyDefRed = {            1,
    0,    0,    0,    0,    0,    5,    8,    9,    0,    0,
    4,    0,    2,    0,    0,   61,    6,    0,   18,   24,
   22,    0,   14,   16,   17,    0,    0,    0,   10,    0,
    0,    0,    0,    0,   15,   25,   23,    0,   87,    0,
   63,   66,   67,   68,   69,   70,   71,    0,    0,    0,
    0,    0,   62,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   26,   29,   30,    0,   64,    0,   43,    0,
    0,    0,   48,    0,    0,    0,    0,    0,   11,    0,
    0,    0,    0,    0,    0,    0,    0,   42,   41,    0,
   34,    0,   37,    0,    0,    0,   59,   60,   58,    0,
   51,    0,   54,    0,    0,    0,   96,    0,  101,   31,
   27,   47,    0,  207,   78,    0,   73,   44,   84,    0,
   33,    0,    0,    0,   39,   46,    0,    0,   50,    0,
    0,    0,   56,    0,    0,    0,    0,   72,    0,    0,
   35,   38,   36,  233,  236,  206,    0,  211,  205,    0,
    0,    0,    0,  181,    0,    0,  208,  209,    0,    0,
  186,  202,   92,    0,    0,  188,    0,    0,    0,    0,
  176,    0,  187,  189,  190,  191,  192,  193,  194,  195,
  196,  197,  198,  199,  200,  201,  203,    0,    0,    0,
   52,   55,   53,    0,  106,  107,    0,  102,   76,    0,
   81,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  218,    0,    0,    0,  216,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  210,   94,    0,    0,  103,   83,  231,    0,
    0,    0,    0,  237,    0,  232,    0,    0,  221,  226,
    0,  224,    0,    0,    0,  227,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  177,  178,  179,    0,
    0,    0,    0,    0,  153,    0,    0,    0,    0,    0,
    0,  161,    0,  108,  110,  111,  112,  113,  114,  115,
  116,  117,  118,  119,  120,  121,  122,    0,    0,    0,
  235,  229,  234,  223,  214,    0,  220,  215,    0,  213,
  217,    0,    0,    0,    0,  155,  140,  138,  212,    0,
  152,    0,    0,   88,    0,    0,    0,    0,   95,  222,
  225,    0,    0,  142,    0,  151,    0,    0,    0,  158,
    0,    0,  146,  129,  126,    0,  109,    0,    0,  131,
  149,    0,    0,    0,  143,    0,    0,    0,  159,    0,
    0,  147,    0,  124,    0,    0,    0,    0,    0,  141,
    0,    0,  134,    0,  154,    0,  145,  128,    0,  130,
    0,  150,    0,  133,    0,    0,  132,    0,    0,  135,
  };
  protected static  short [] yyDgoto  = {             1,
    5,    2,   13,    6,    7,    8,   16,   29,   34,   62,
   22,   23,   24,   25,   26,   63,   64,   65,   74,   96,
   90,   91,   92,   93,   94,  281,  160,   76,  100,  101,
  102,  103,  104,   27,   40,   41,   42,   43,   44,   45,
   46,   47,   48,  138,   84,  117,  161,  162,   49,  201,
  282,   50,  128,  190,  283,   51,  107,  108,  109,  197,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,  297,  346,  364,  164,  165,  253,  372,
  373,  318,  333,  334,  352,  166,  342,  343,  338,  339,
  340,  167,  168,  169,  170,  171,  172,  208,  173,  174,
  175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
  185,  186,  187,  188,  218,  248,  249,
  };
  protected static  short [] yySindex = {            0,
    0,   12, -123, -239, -244,    0,    0,    0, -216, -130,
    0,   12,    0, -130, -284,    0,    0,  -98,    0,    0,
    0,  -99,    0,    0,    0,  -91,  -73, -284,    0, -284,
 -337, -266,  -99,   -6,    0,    0,    0,    0,    0, -219,
    0,    0,    0,    0,    0,    0,    0,   13, -209,  -66,
   26,  -45,    0, -266,  -41,  -39,   77, -300,   39, -284,
 -300, -218,    0,    0,    0,   39,    0, -217,    0,  -33,
  -39, -162,    0,   40,   54,   40,   23,   40,    0,  -45,
   40, -284,   34,   90,   42, -284,    4,    0,    0,  -54,
    0,   51,    0,   60, -284,  127,    0,    0,    0,  -24,
    0,   52,    0,   71,  127,   83,    0,  110,    0,    0,
    0,    0,   89,    0,    0,  -39,    0,    0,    0, -284,
    0, -162, -162, -284,    0,    0,  -74,  146,    0,   54,
   54, -284,    0,  146,  -31,   23, -270,    0,  117,   97,
    0,    0,    0,    0,    0,    0,   88,    0,    0,  134,
  140,  141,  -74,    0, -284,  133,    0,    0,  -94,  142,
    0,    0,    0,   36, -127,    0, -251,   92,   93,   41,
    0,  124,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  105,  -74,  181,
    0,    0,    0,  182,    0,    0,  106,    0,    0,  -74,
    0,  142,  113,  -74,  -74,  -74,  150,  133,  142,  124,
 -162,  111, -162,    0,  -74,  112,  114,    0,  -74,  -74,
  -74,  -74,  -74,  -74,  -74,  -74,  -74,  -74,  -74,  -74,
  -74,  133,    0,    0,   53,   53,    0,    0,    0,  116,
  153,  157,  158,    0,  124,    0,  -74,   50,    0,    0,
   94,    0, -129, -196,  142,    0,   92,   92,   93,   93,
   93,   93,   93,   93,   41,   41,    0,    0,    0,   36,
  -74,  -74,   53,   53,    0,  -74,  -74,  120,  -74,  -63,
   65,    0, -212,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   91,  -90, -168,
    0,    0,    0,    0,    0, -162,    0,    0,  -74,    0,
    0,  166,  173, -147, -286,    0,    0,    0,    0,  168,
    0,  -74, -284,    0,   53,  -74,  125,  -74,    0,    0,
    0,  -74, -213,    0,   53,    0, -284,  215,  171,    0,
 -284, -125,    0,    0,    0,  139,    0,   79,  -68,    0,
    0, -192,   53,  217,    0, -255,  183,  224,    0,  187,
  226,    0,  -74,    0,  151,  -74,   53,  -74,  174,    0,
  -74,  227,    0,   53,    0,   53,    0,    0,  -74,    0,
  174,    0,  193,    0,  174,  174,    0,   53, -255,    0,
  };
  protected static  short [] yyRindex = {            0,
    0,    1,    0,    0,  508,    0,    0,    0,    0,  230,
    0,    1,    0, -277,    0,    0,    0,  231,    0,    0,
    0, -257,    0,    0,    0,  -22,    0,    0,    0,    0,
    0, -261,  243,    0,    0,    0,    0, -250,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -141,    0, -261,    0,    0,    0,   -5,   59,    0,
 -259,    0,    0,    0,    0, -223,    0, -144,    0,    0,
    0,  149,    0,  -75,  170,  -75, -132, -128,    0, -141,
 -128,    0,    0, -115,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  -78,    0,    0,    0,    0,
    0,    0,    0,    0,  -78,    0,    0, -106,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  149,  149,    0,    0,    0,    0,  245,    0,  170,
  170,    0,    0,  245, -260,    0,    0,    0, -105, -103,
    0,    0,    0,    0,    0,    0,  252,    0,    0,    0,
  319,    0,    0,    0,    0,    0,    0,    0,  386,  453,
    0,    0,    0,  587,  453,    0,  -92, 1581, 1179,  950,
    0,  721,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  520,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  453,  185,    0,    0,    0,    0,    0,  453,  788,
   72,    0,   72,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  -96,  -96,    0,    0,    0,    0,
    0,    0,    0,    0,  855,    0,    0,    0,    0,    0,
    0,    0,    0,  453,  453,    0, 1603, 1644, 1215, 1282,
 1318, 1385, 1421, 1488, 1017, 1084,    0,    0,    0,  654,
    0,    0,  -96, -133,    0,    0, -178,    0, -178,  883,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1514,  922,    0,
    0,    0,    0,    0,    0,   72,    0,    0,    0,    0,
    0,    0,    0,    0,  -19,    0,    0,    0,    0,    0,
    0,    0, -208,    0, -157,    0,    0,    0,    0,    0,
    0,    0,  261,    0, -226,    0,    0,  261,  147,    0,
    0,  261,    0,    0,    0, -109,    0,    0,  922,    0,
    0,    0,  -96,    0,    0,  261,    0,    0,    0,    0,
    0,    0,    0,    0, 1112,    0, -133,    0,  262,    0,
    0,    0,    0, -133,    0, -133,    0,    0,    0,    0,
 -143,    0,    0,    0,  -60,  -56,    0, -226,  261,    0,
  };
  protected static  short [] yyGindex = {            0,
    0,    0,    0,  527,    0,    0,  529,    0,    0,    0,
  513,  -30,  397,    0,  398,  466,    0,    0,  487,  264,
    0,  427,    0,  429, -205,  -29,  -48,  484,    0,  423,
    0,  424,    0,    0,    0,  500,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  419,  474,    0,    0,
 -120,    0,  454,  426, -198,    0,    0,    0,  422,    0,
  237,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, -214, -195,  238,  172,
   28,  284,    0,  232,    0,  289,    0,  228,    0,    0,
  229,    0,  236,  179,  235,  123, -119,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -155,  356,  265,
  };
  protected static  short [] yyTable = {            35,
    7,   59,   85,   66,  214,  247,  163,  247,  146,  219,
   12,   74,   11,   86,   65,  104,   32,  270,  353,  371,
  298,  298,   75,   72,   97,   19,   70,   36,   37,   77,
   13,   66,  207,  337,   38,   98,  210,  300,   39,  299,
  299,   87,   73,  220,  149,  325,  239,  123,  123,  123,
   85,  113,   49,  246,   56,  119,   53,   79,  298,  298,
  353,   86,   12,  324,  126,  125,  125,  125,  234,  104,
   65,  104,   32,   32,  314,  315,  325,  299,  299,  238,
   20,   21,   13,  241,  242,  243,  139,   12,  245,  140,
  157,  158,  114,  143,  252,  139,  139,  139,  310,  311,
  247,  193,   57,   89,  105,  123,  332,  329,   49,   49,
  298,  125,   54,   80,   90,   82,  123,  123,  123,  325,
  298,  367,   88,  125,  209,   10,  304,  213,  336,  299,
  144,   77,  144,   89,   28,   83,  356,  368,  298,  299,
  123,  139,  123,   99,  125,    9,   73,   45,  353,   14,
  312,  313,  298,  139,  369,  316,  317,  299,  317,  298,
   79,  298,  123,  325,  127,  127,  127,  255,  381,  100,
   80,  299,   82,  298,  123,  385,  144,  386,  299,  123,
  299,  162,  162,  162,  325,   77,  123,   77,  331,  389,
   28,  144,  299,  310,  341,  162,  213,  308,  123,   99,
  309,  344,  145,   45,  146,  252,  162,  350,   15,   91,
  127,  351,   45,  160,   32,  160,   79,  148,  147,  148,
   91,  162,  127,   45,   45,  100,   80,  162,   82,  211,
   30,  217,  162,  213,  162,  123,  148,  162,   21,  162,
  149,   28,  378,  150,  151,  380,  152,  382,   73,  153,
  383,   21,   21,   21,  156,  213,  156,    7,  387,  160,
  211,  154,  328,  148,  212,   21,  155,   31,  217,  195,
  121,    3,   21,  196,   73,  122,   21,   21,  255,   73,
    4,   52,   32,  156,  366,   55,  157,  158,  114,  322,
  159,   21,  345,   32,   32,  212,   85,   21,   58,   86,
  129,   21,   21,   21,   21,  130,  357,   21,   21,   21,
  360,   60,   21,   21,   21,   21,   21,   21,  144,   61,
  271,   21,   21,   68,   21,   69,   21,   32,   21,  145,
   21,  146,    7,   85,   21,  272,  120,   40,   97,  105,
   71,  110,   21,  273,  112,  147,   49,   98,   40,   99,
   40,  267,  268,  269,  274,  275,  276,   49,   49,  277,
  354,  215,   75,  148,   40,  358,  216,  149,  278,  361,
  150,  151,   95,  152,  305,  279,  153,  229,  230,  306,
  123,  131,   40,  124,  132,  231,   40,  106,  154,   40,
   40,   49,   40,  155,   85,   40,  114,  323,  144,  259,
  260,  261,  262,  263,  264,  365,  118,   40,  309,  145,
  156,  146,   40,  157,  158,  114,  326,  280,  307,  116,
  157,  327,  157,  306,  125,  147,  127,  227,  228,   40,
  221,  222,   40,   40,   40,  133,   40,  135,  223,  136,
  224,  137,  225,  148,  189,   21,   85,  149,  226,  200,
  150,  151,   20,  152,  257,  258,  153,  204,   21,   21,
   21,  265,  266,  205,  206,  213,  233,  232,  235,  236,
  237,  240,   21,  155,  244,  250,  254,  301,  256,   21,
   36,  302,  303,   21,  319,  332,  335,  341,  353,  349,
  337,  363,  370,  157,  158,  114,  374,  159,   21,  375,
  376,  377,  384,  379,   21,  325,  388,    3,   21,   21,
   21,   21,  230,   40,   21,   21,   21,   12,   19,   21,
   21,   21,   21,   21,   21,  230,  230,  230,   21,   21,
   20,   21,   93,   21,   57,   21,  136,  137,   17,  230,
   33,   21,   18,  202,  203,  111,  230,   78,  141,   81,
  230,  142,  191,   67,  192,  199,  115,  198,  134,  194,
  390,  347,  321,  348,  355,  230,  320,  359,  251,  362,
  330,  230,    0,    0,    0,    0,  230,  230,  230,  228,
    0,  230,  230,  230,    0,    0,  230,  230,  230,  230,
  230,  230,  228,  228,  228,  230,  230,    0,  230,    0,
  230,    0,  230,    0,    0,    0,  228,    0,  230,    0,
    0,    0,    0,  228,    0,    0,    0,  228,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  228,    0,    0,    0,    0,    0,  228,    0,
    0,    0,    0,  228,  228,  228,  212,    0,  228,  228,
  228,    0,    0,  228,  228,  228,  228,  228,  228,  212,
  212,  212,  228,  228,    0,  228,    0,  228,    0,  228,
    0,    0,    0,  212,    0,  228,    0,    0,    0,    0,
  212,    0,    0,    0,  212,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  212,
    0,    0,    0,    0,    0,  212,    0,    0,    0,    0,
  212,  212,  212,  219,    0,  212,  212,  212,    0,    0,
  212,  212,  212,  212,  212,  212,  219,  219,  219,  212,
  212,    0,  212,    0,  212,    0,  212,    0,    0,    0,
  219,    0,  212,    0,    0,    0,    0,  219,    0,    0,
    0,  219,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  219,    0,    0,    0,
    0,    0,  219,    0,    0,    0,    0,  219,  219,  219,
  204,    0,  219,  219,  219,    0,    0,  219,  219,  219,
  219,  219,  219,  204,  204,  204,  219,  219,    0,  219,
    0,  219,    0,  219,    0,    0,    0,  204,    0,  219,
    0,    0,    0,    0,  204,    0,    0,    0,  204,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  204,    0,    0,    0,    0,    0,  204,
    0,    0,    0,    0,  204,  204,  204,  184,    0,  204,
  204,  204,    0,    0,  204,  204,  204,  204,  204,  204,
  184,  184,  184,  204,  204,    0,  204,    0,  204,    0,
  204,    0,    0,    0,  184,    0,  204,    0,    0,    0,
    0,  184,    0,    0,    0,  184,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  184,    0,    0,    0,    0,    0,  184,    0,    0,    0,
    0,  184,    0,  184,  185,    0,  184,    0,  184,    0,
    0,  184,  184,  184,  184,  184,  184,  185,  185,  185,
  184,  184,    0,  184,    0,  184,    0,  184,    0,    0,
    0,  185,    0,  184,    0,    0,    0,    0,  185,    0,
    0,    0,  185,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  185,    0,    0,
    0,    0,    0,  185,    0,    0,    0,    0,  185,    0,
  185,  180,    0,  185,    0,  185,    0,    0,  185,  185,
  185,  185,  185,  185,  180,  180,  180,  185,  185,    0,
  185,    0,  185,    0,  185,    0,    0,    0,  180,    0,
  185,    0,    0,    0,    0,  180,    0,    0,    0,  180,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  180,    0,    0,    0,    0,    0,
  180,    0,    0,    0,    0,  180,    0,  180,  183,    0,
  180,    0,  180,    0,    0,  180,  180,  180,  180,  180,
  180,  183,  183,  183,    0,  180,    0,  180,    0,  180,
    0,  180,    0,    0,    0,  183,    0,  180,    0,    0,
    0,    0,  183,    0,    0,    0,  183,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  183,    0,    0,    0,    0,    0,  183,    0,    0,
    0,    0,  183,    0,  183,  182,    0,  183,    0,  183,
    0,    0,  183,  183,  183,  183,  183,  183,  182,  182,
  182,    0,  183,    0,  183,    0,  183,    0,  183,    0,
    0,    0,  182,  212,  183,    0,    0,    0,    0,  182,
    0,    0,    0,  182,    0,    0,  212,  212,  212,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  182,    0,
    0,    0,    0,    0,  182,    0,    0,  212,    0,  182,
    0,  182,  219,    0,  182,    0,  182,    0,    0,  182,
  182,  182,  182,  182,  182,  219,  219,  219,    0,  182,
    0,  182,  212,  182,    0,  182,    0,    0,  212,    0,
  173,  182,   43,  212,  212,   43,  219,  212,  212,  212,
  212,  212,  212,  173,  173,  173,  212,  212,    0,  212,
    0,  212,    0,  212,    0,    0,    0,  173,    0,  212,
    0,  219,    0,    0,  173,    0,    0,  219,  173,    0,
    0,    0,  219,  219,    0,    0,  219,  219,  219,  219,
  219,  219,    0,  173,    0,  219,  219,    0,  219,  173,
  219,    0,  219,    0,  173,    0,  173,  174,  219,  173,
    0,  173,    0,    0,  173,  173,    0,    0,  173,  173,
  174,  174,  174,    0,    0,    0,  173,    0,  173,    0,
  173,    0,    0,    0,  174,    0,  173,    0,    0,    0,
    0,  174,    0,    0,    0,  174,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  174,    0,    0,    0,    0,    0,  174,    0,    0,    0,
    0,  174,    0,  174,  175,    0,  174,    0,  174,    0,
    0,  174,  174,    0,    0,  174,  174,  175,  175,  175,
    0,    0,    0,  174,    0,  174,    0,  174,    0,    0,
    0,  175,  215,  174,    0,    0,    0,    0,  175,    0,
    0,    0,  175,    0,    0,  215,  215,  215,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  175,    0,    0,
    0,    0,    0,  175,    0,    0,  215,    0,  175,    0,
  175,    0,    0,  175,    0,  175,    0,    0,  175,  175,
    0,    0,  175,  175,    0,    0,    0,    0,    0,    0,
  175,  215,  175,    0,  175,    0,    0,  215,    0,  166,
  175,    0,  215,  215,    0,    0,  215,  215,  215,  215,
  215,  215,  166,  166,  166,  215,  215,    0,  215,    0,
  215,    0,  215,    0,    0,    0,  166,    0,  215,    0,
    0,    0,    0,  166,    0,  169,    0,  166,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  169,  169,
  169,    0,  166,    0,    0,    0,    0,    0,  166,    0,
    0,    0,  169,  166,    0,  166,    0,    0,  166,  169,
  166,    0,    0,  169,    0,    0,    0,  166,  166,    0,
    0,    0,    0,    0,    0,  166,    0,  166,  169,  166,
    0,    0,    0,    0,  169,  166,    0,    0,    0,  169,
    0,  169,  171,    0,  169,    0,  169,    0,    0,    0,
    0,    0,    0,  169,  169,  171,  171,  171,    0,    0,
    0,  169,    0,  169,    0,  169,    0,    0,    0,  171,
    0,  169,    0,    0,    0,    0,  171,    0,  168,    0,
  171,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  168,  168,  168,    0,  171,    0,    0,    0,    0,
    0,  171,    0,    0,    0,  168,  171,    0,  171,    0,
    0,  171,  168,  171,    0,    0,  168,    0,    0,    0,
  171,  171,    0,    0,    0,    0,    0,    0,  171,    0,
  171,  168,  171,    0,    0,    0,    0,  168,  171,    0,
    0,    0,  168,    0,  168,  170,    0,  168,    0,  168,
    0,    0,    0,    0,    0,    0,  168,  168,  170,  170,
  170,    0,    0,    0,  168,    0,  168,    0,  168,    0,
    0,    0,  170,    0,  168,    0,    0,    0,    0,  170,
    0,  172,    0,  170,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  172,  172,  172,    0,  170,    0,
    0,    0,    0,    0,  170,    0,    0,    0,  172,  170,
    0,  170,    0,    0,  170,  172,  170,    0,    0,  172,
    0,    0,    0,  170,  170,    0,    0,    0,    0,    0,
    0,  170,    0,  170,  172,  170,    0,    0,    0,    0,
  172,  170,    0,    0,    0,  172,    0,  172,  167,    0,
  172,    0,  172,    0,    0,    0,    0,    0,    0,  172,
  172,  167,  167,  167,    0,    0,    0,  172,    0,  172,
    0,  172,    0,    0,  184,  167,    0,  172,    0,    0,
    0,    0,  167,    0,    0,    0,  167,  184,  184,  184,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  167,    0,    0,    0,    0,    0,  167,  184,    0,
    0,    0,  167,    0,  167,    0,    0,  167,    0,  167,
    0,    0,    0,    0,    0,    0,  167,  167,    0,    0,
    0,    0,    0,  184,  167,    0,  167,    0,  167,    0,
    0,  163,    0,    0,  167,  184,    0,    0,  184,  184,
  184,  184,  184,  184,  163,  163,  163,  184,  184,    0,
  184,    0,  184,  164,  184,    0,    0,    0,  163,    0,
  184,    0,    0,    0,    0,  163,  164,  164,  164,  163,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  164,    0,    0,    0,  163,    0,    0,  164,    0,    0,
  163,  164,    0,    0,  165,  163,    0,  163,    0,    0,
  163,    0,  163,    0,    0,    0,  164,  165,  165,  165,
    0,    0,  164,    0,    0,    0,    0,  164,    0,  164,
    0,  165,  164,    0,  164,    0,    0,    0,  165,    0,
    0,    0,  165,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  165,    0,    0,
    0,    0,    0,  165,    0,    0,    0,    0,  165,    0,
  165,    0,    0,  165,    0,  165,
  };
  protected static  short [] yyCheck = {            30,
    0,   50,  264,   52,  160,  211,  127,  213,  279,  261,
  288,  273,  257,  264,  276,  276,  276,  232,  274,  275,
  235,  236,  273,  324,  286,  310,   56,  365,  366,   60,
  288,   80,  153,  320,  301,  286,  156,  236,  305,  235,
  236,   71,  343,  295,  315,  332,  202,  274,  275,  276,
  312,   82,  276,  209,  264,   86,  276,  276,  273,  274,
  274,  312,  340,  276,   95,  274,  275,  276,  189,  330,
  332,  332,  332,  333,  273,  274,  332,  273,  274,  200,
  365,  366,  340,  204,  205,  206,  116,  332,  208,  120,
  361,  362,  363,  124,  215,  274,  275,  276,  254,  255,
  306,  132,  312,  365,  365,  332,  320,  276,  332,  333,
  325,  320,  332,  332,  365,  333,  274,  275,  276,  332,
  335,  314,  285,  332,  155,  365,  247,  324,  276,  325,
  274,  276,  276,  296,  276,  353,  335,  330,  353,  335,
  274,  320,  276,  276,  353,  269,  343,  276,  274,  366,
  271,  272,  367,  332,  353,  276,  277,  353,  279,  374,
  276,  376,  320,  332,  274,  275,  276,  216,  367,  276,
  276,  367,  276,  388,  332,  374,  320,  376,  374,  276,
  376,  274,  275,  276,  332,  330,  320,  332,  309,  388,
  332,  266,  388,  349,  320,  288,  324,  327,  332,  332,
  330,  322,  277,  332,  279,  326,  299,  328,  339,  288,
  320,  332,  288,  274,  288,  276,  332,  274,  293,  276,
  299,  314,  332,  299,  300,  332,  332,  320,  332,  324,
  330,  359,  325,  324,  327,  332,  311,  330,  261,  332,
  315,  340,  363,  318,  319,  366,  321,  368,  343,  324,
  371,  274,  275,  276,  274,  324,  276,  257,  379,  320,
  324,  336,  353,  320,  359,  288,  341,  359,  359,  301,
  325,  260,  295,  305,  343,  330,  299,  300,  327,  343,
  269,  288,  288,  358,  353,  273,  361,  362,  363,  353,
  365,  314,  323,  299,  300,  359,  330,  320,  365,  333,
  325,  324,  325,  326,  327,  330,  337,  330,  331,  332,
  341,  286,  335,  336,  337,  338,  339,  340,  266,  365,
  268,  344,  345,  365,  347,  365,  349,  333,  351,  277,
  353,  279,  332,  330,  357,  283,  333,  266,  285,   76,
  264,   78,  365,  291,   81,  293,  288,  294,  277,  296,
  279,  229,  230,  231,  302,  303,  304,  299,  300,  307,
  333,  326,  324,  311,  293,  338,  331,  315,  316,  342,
  318,  319,  333,  321,  325,  323,  324,  337,  338,  330,
  330,  330,  311,  333,  333,  345,  315,  365,  336,  318,
  319,  333,  321,  341,  330,  324,  363,  333,  266,  221,
  222,  223,  224,  225,  226,  327,  365,  336,  330,  277,
  358,  279,  341,  361,  362,  363,  326,  365,  325,  330,
  274,  331,  276,  330,  365,  293,  300,  335,  336,  358,
  339,  340,  361,  362,  363,  365,  365,  355,  347,  330,
  349,  353,  351,  311,  299,  261,  330,  315,  357,  353,
  318,  319,  365,  321,  219,  220,  324,  324,  274,  275,
  276,  227,  228,  324,  324,  324,  362,  344,  288,  288,
  365,  359,  288,  341,  325,  365,  365,  325,  365,  295,
  365,  325,  325,  299,  365,  320,  314,  320,  274,  365,
  320,  353,  276,  361,  362,  363,  314,  365,  314,  276,
  314,  276,  276,  353,  320,  332,  314,    0,  324,  325,
  326,  327,  261,  365,  330,  331,  332,  288,  288,  335,
  336,  337,  338,  339,  340,  274,  275,  276,  344,  345,
  288,  347,  288,  349,  365,  351,  276,  276,   12,  288,
   28,  357,   14,  147,  147,   80,  295,   61,  122,   66,
  299,  123,  130,   54,  131,  137,   83,  136,  105,  134,
  389,  325,  279,  326,  333,  314,  278,  339,  213,  342,
  306,  320,   -1,   -1,   -1,   -1,  325,  326,  327,  261,
   -1,  330,  331,  332,   -1,   -1,  335,  336,  337,  338,
  339,  340,  274,  275,  276,  344,  345,   -1,  347,   -1,
  349,   -1,  351,   -1,   -1,   -1,  288,   -1,  357,   -1,
   -1,   -1,   -1,  295,   -1,   -1,   -1,  299,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  314,   -1,   -1,   -1,   -1,   -1,  320,   -1,
   -1,   -1,   -1,  325,  326,  327,  261,   -1,  330,  331,
  332,   -1,   -1,  335,  336,  337,  338,  339,  340,  274,
  275,  276,  344,  345,   -1,  347,   -1,  349,   -1,  351,
   -1,   -1,   -1,  288,   -1,  357,   -1,   -1,   -1,   -1,
  295,   -1,   -1,   -1,  299,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  314,
   -1,   -1,   -1,   -1,   -1,  320,   -1,   -1,   -1,   -1,
  325,  326,  327,  261,   -1,  330,  331,  332,   -1,   -1,
  335,  336,  337,  338,  339,  340,  274,  275,  276,  344,
  345,   -1,  347,   -1,  349,   -1,  351,   -1,   -1,   -1,
  288,   -1,  357,   -1,   -1,   -1,   -1,  295,   -1,   -1,
   -1,  299,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,   -1,   -1,
   -1,   -1,  320,   -1,   -1,   -1,   -1,  325,  326,  327,
  261,   -1,  330,  331,  332,   -1,   -1,  335,  336,  337,
  338,  339,  340,  274,  275,  276,  344,  345,   -1,  347,
   -1,  349,   -1,  351,   -1,   -1,   -1,  288,   -1,  357,
   -1,   -1,   -1,   -1,  295,   -1,   -1,   -1,  299,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  314,   -1,   -1,   -1,   -1,   -1,  320,
   -1,   -1,   -1,   -1,  325,  326,  327,  261,   -1,  330,
  331,  332,   -1,   -1,  335,  336,  337,  338,  339,  340,
  274,  275,  276,  344,  345,   -1,  347,   -1,  349,   -1,
  351,   -1,   -1,   -1,  288,   -1,  357,   -1,   -1,   -1,
   -1,  295,   -1,   -1,   -1,  299,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  314,   -1,   -1,   -1,   -1,   -1,  320,   -1,   -1,   -1,
   -1,  325,   -1,  327,  261,   -1,  330,   -1,  332,   -1,
   -1,  335,  336,  337,  338,  339,  340,  274,  275,  276,
  344,  345,   -1,  347,   -1,  349,   -1,  351,   -1,   -1,
   -1,  288,   -1,  357,   -1,   -1,   -1,   -1,  295,   -1,
   -1,   -1,  299,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,   -1,
   -1,   -1,   -1,  320,   -1,   -1,   -1,   -1,  325,   -1,
  327,  261,   -1,  330,   -1,  332,   -1,   -1,  335,  336,
  337,  338,  339,  340,  274,  275,  276,  344,  345,   -1,
  347,   -1,  349,   -1,  351,   -1,   -1,   -1,  288,   -1,
  357,   -1,   -1,   -1,   -1,  295,   -1,   -1,   -1,  299,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  314,   -1,   -1,   -1,   -1,   -1,
  320,   -1,   -1,   -1,   -1,  325,   -1,  327,  261,   -1,
  330,   -1,  332,   -1,   -1,  335,  336,  337,  338,  339,
  340,  274,  275,  276,   -1,  345,   -1,  347,   -1,  349,
   -1,  351,   -1,   -1,   -1,  288,   -1,  357,   -1,   -1,
   -1,   -1,  295,   -1,   -1,   -1,  299,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  314,   -1,   -1,   -1,   -1,   -1,  320,   -1,   -1,
   -1,   -1,  325,   -1,  327,  261,   -1,  330,   -1,  332,
   -1,   -1,  335,  336,  337,  338,  339,  340,  274,  275,
  276,   -1,  345,   -1,  347,   -1,  349,   -1,  351,   -1,
   -1,   -1,  288,  261,  357,   -1,   -1,   -1,   -1,  295,
   -1,   -1,   -1,  299,   -1,   -1,  274,  275,  276,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,
   -1,   -1,   -1,   -1,  320,   -1,   -1,  295,   -1,  325,
   -1,  327,  261,   -1,  330,   -1,  332,   -1,   -1,  335,
  336,  337,  338,  339,  340,  274,  275,  276,   -1,  345,
   -1,  347,  320,  349,   -1,  351,   -1,   -1,  326,   -1,
  261,  357,  330,  331,  332,  333,  295,  335,  336,  337,
  338,  339,  340,  274,  275,  276,  344,  345,   -1,  347,
   -1,  349,   -1,  351,   -1,   -1,   -1,  288,   -1,  357,
   -1,  320,   -1,   -1,  295,   -1,   -1,  326,  299,   -1,
   -1,   -1,  331,  332,   -1,   -1,  335,  336,  337,  338,
  339,  340,   -1,  314,   -1,  344,  345,   -1,  347,  320,
  349,   -1,  351,   -1,  325,   -1,  327,  261,  357,  330,
   -1,  332,   -1,   -1,  335,  336,   -1,   -1,  339,  340,
  274,  275,  276,   -1,   -1,   -1,  347,   -1,  349,   -1,
  351,   -1,   -1,   -1,  288,   -1,  357,   -1,   -1,   -1,
   -1,  295,   -1,   -1,   -1,  299,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  314,   -1,   -1,   -1,   -1,   -1,  320,   -1,   -1,   -1,
   -1,  325,   -1,  327,  261,   -1,  330,   -1,  332,   -1,
   -1,  335,  336,   -1,   -1,  339,  340,  274,  275,  276,
   -1,   -1,   -1,  347,   -1,  349,   -1,  351,   -1,   -1,
   -1,  288,  261,  357,   -1,   -1,   -1,   -1,  295,   -1,
   -1,   -1,  299,   -1,   -1,  274,  275,  276,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,   -1,
   -1,   -1,   -1,  320,   -1,   -1,  295,   -1,  325,   -1,
  327,   -1,   -1,  330,   -1,  332,   -1,   -1,  335,  336,
   -1,   -1,  339,  340,   -1,   -1,   -1,   -1,   -1,   -1,
  347,  320,  349,   -1,  351,   -1,   -1,  326,   -1,  261,
  357,   -1,  331,  332,   -1,   -1,  335,  336,  337,  338,
  339,  340,  274,  275,  276,  344,  345,   -1,  347,   -1,
  349,   -1,  351,   -1,   -1,   -1,  288,   -1,  357,   -1,
   -1,   -1,   -1,  295,   -1,  261,   -1,  299,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  274,  275,
  276,   -1,  314,   -1,   -1,   -1,   -1,   -1,  320,   -1,
   -1,   -1,  288,  325,   -1,  327,   -1,   -1,  330,  295,
  332,   -1,   -1,  299,   -1,   -1,   -1,  339,  340,   -1,
   -1,   -1,   -1,   -1,   -1,  347,   -1,  349,  314,  351,
   -1,   -1,   -1,   -1,  320,  357,   -1,   -1,   -1,  325,
   -1,  327,  261,   -1,  330,   -1,  332,   -1,   -1,   -1,
   -1,   -1,   -1,  339,  340,  274,  275,  276,   -1,   -1,
   -1,  347,   -1,  349,   -1,  351,   -1,   -1,   -1,  288,
   -1,  357,   -1,   -1,   -1,   -1,  295,   -1,  261,   -1,
  299,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  274,  275,  276,   -1,  314,   -1,   -1,   -1,   -1,
   -1,  320,   -1,   -1,   -1,  288,  325,   -1,  327,   -1,
   -1,  330,  295,  332,   -1,   -1,  299,   -1,   -1,   -1,
  339,  340,   -1,   -1,   -1,   -1,   -1,   -1,  347,   -1,
  349,  314,  351,   -1,   -1,   -1,   -1,  320,  357,   -1,
   -1,   -1,  325,   -1,  327,  261,   -1,  330,   -1,  332,
   -1,   -1,   -1,   -1,   -1,   -1,  339,  340,  274,  275,
  276,   -1,   -1,   -1,  347,   -1,  349,   -1,  351,   -1,
   -1,   -1,  288,   -1,  357,   -1,   -1,   -1,   -1,  295,
   -1,  261,   -1,  299,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  274,  275,  276,   -1,  314,   -1,
   -1,   -1,   -1,   -1,  320,   -1,   -1,   -1,  288,  325,
   -1,  327,   -1,   -1,  330,  295,  332,   -1,   -1,  299,
   -1,   -1,   -1,  339,  340,   -1,   -1,   -1,   -1,   -1,
   -1,  347,   -1,  349,  314,  351,   -1,   -1,   -1,   -1,
  320,  357,   -1,   -1,   -1,  325,   -1,  327,  261,   -1,
  330,   -1,  332,   -1,   -1,   -1,   -1,   -1,   -1,  339,
  340,  274,  275,  276,   -1,   -1,   -1,  347,   -1,  349,
   -1,  351,   -1,   -1,  261,  288,   -1,  357,   -1,   -1,
   -1,   -1,  295,   -1,   -1,   -1,  299,  274,  275,  276,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  314,   -1,   -1,   -1,   -1,   -1,  320,  295,   -1,
   -1,   -1,  325,   -1,  327,   -1,   -1,  330,   -1,  332,
   -1,   -1,   -1,   -1,   -1,   -1,  339,  340,   -1,   -1,
   -1,   -1,   -1,  320,  347,   -1,  349,   -1,  351,   -1,
   -1,  261,   -1,   -1,  357,  332,   -1,   -1,  335,  336,
  337,  338,  339,  340,  274,  275,  276,  344,  345,   -1,
  347,   -1,  349,  261,  351,   -1,   -1,   -1,  288,   -1,
  357,   -1,   -1,   -1,   -1,  295,  274,  275,  276,  299,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  288,   -1,   -1,   -1,  314,   -1,   -1,  295,   -1,   -1,
  320,  299,   -1,   -1,  261,  325,   -1,  327,   -1,   -1,
  330,   -1,  332,   -1,   -1,   -1,  314,  274,  275,  276,
   -1,   -1,  320,   -1,   -1,   -1,   -1,  325,   -1,  327,
   -1,  288,  330,   -1,  332,   -1,   -1,   -1,  295,   -1,
   -1,   -1,  299,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,   -1,
   -1,   -1,   -1,  320,   -1,   -1,   -1,   -1,  325,   -1,
  327,   -1,   -1,  330,   -1,  332,
  };

#line 1508 "parser.jay"

    public virtual LexicalAnalyzer Lexer
    {
        get { return lexer; }
    }

    public virtual void Parse()
    {
        try {
            yyparse(lexer);
        }
        catch (Exception e) {
            report.Error(lexer.Location, e.Message);
            throw;
        }
    }

    protected virtual void AddClass(ClassDefinition cls)
    {
        sourceFile.AddChild(cls);
        ClassDefinition c = program.TypeManager.GetClass(cls.Name);
        if (c != null) {
            report.Error(cls.Location,
                         "redefinition of class {0}\n" +
                         "\tthe previous definition is at:{1}:{2}:{3}",
                         cls.Name,
                         c.Location.FileName,
                         c.Location.Line,
                         c.Location.Column);
            return;
        }
        program.TypeManager.AddClass(cls);
    }

    protected virtual ArrayList NewList(Object obj)
    {
        if (obj is ArrayList)
            return (ArrayList) obj;
        ArrayList result = new ArrayList();
        if (obj == null)
            return result;
        result.Add(obj);
        return result;
    }

    protected virtual ArrayList ListAppend(Object list, Object obj)
    {
        ArrayList result = (ArrayList) list;
        if (result == null)
            result = new ArrayList();
        if (obj == null)
            return result;
        if (obj is ArrayList) {
            result.AddRange((ArrayList) obj);
        }
        else {
            result.Add(obj);
        }
        return result;
    }

    protected virtual ModalExpression InArg(Expression expr)
    {
        return new ModalExpression(ArgumentMode.In, expr,
                                   expr.Location);
    }

    protected virtual TypedNodeList OneInArg(Expression expr)
    {
        return new TypedNodeList(InArg(expr));
    }
}
#line default
namespace yydebug {
        using System;
	 public interface yyDebug {
		 void push (int state, Object value);
		 void lex (int state, int token, string name, Object value);
		 void shift (int from, int to, int errorFlag);
		 void pop (int state);
		 void discard (int state, int token, string name, Object value);
		 void reduce (int from, int to, int rule, string text, int len);
		 void shift (int from, int to);
		 void accept (Object value);
		 void error (string message);
		 void reject ();
	 }
	 
	 class yyDebugSimple : yyDebug {
		 void println (string s){
			 Console.WriteLine (s);
		 }
		 
		 public void push (int state, Object value) {
			 println ("push\tstate "+state+"\tvalue "+value);
		 }
		 
		 public void lex (int state, int token, string name, Object value) {
			 println("lex\tstate "+state+"\treading "+name+"\tvalue "+value);
		 }
		 
		 public void shift (int from, int to, int errorFlag) {
			 switch (errorFlag) {
			 default:				// normally
				 println("shift\tfrom state "+from+" to "+to);
				 break;
			 case 0: case 1: case 2:		// in error recovery
				 println("shift\tfrom state "+from+" to "+to
					     +"\t"+errorFlag+" left to recover");
				 break;
			 case 3:				// normally
				 println("shift\tfrom state "+from+" to "+to+"\ton error");
				 break;
			 }
		 }
		 
		 public void pop (int state) {
			 println("pop\tstate "+state+"\ton error");
		 }
		 
		 public void discard (int state, int token, string name, Object value) {
			 println("discard\tstate "+state+"\ttoken "+name+"\tvalue "+value);
		 }
		 
		 public void reduce (int from, int to, int rule, string text, int len) {
			 println("reduce\tstate "+from+"\tuncover "+to
				     +"\trule ("+rule+") "+text);
		 }
		 
		 public void shift (int from, int to) {
			 println("goto\tfrom state "+from+" to "+to);
		 }
		 
		 public void accept (Object value) {
			 println("accept\tvalue "+value);
		 }
		 
		 public void error (string message) {
			 println("error\t"+message);
		 }
		 
		 public void reject () {
			 println("reject");
		 }
		 
	 }
}
// %token constants
 class Token {
  public const int EOF = 257;
  public const int NONE = 258;
  public const int ERROR = 259;
  public const int ABSTRACT = 260;
  public const int AND = 261;
  public const int ANY = 262;
  public const int ASSERT = 263;
  public const int ATTR = 264;
  public const int BIND = 265;
  public const int BREAK_BANG = 266;
  public const int BUILTIN = 267;
  public const int CASE = 268;
  public const int CLASS = 269;
  public const int CLUSTERS = 270;
  public const int CLUSTERS_BANG = 271;
  public const int COHORT = 272;
  public const int CONST = 273;
  public const int ELSE = 274;
  public const int ELSIF = 275;
  public const int END = 276;
  public const int EXCEPTION = 277;
  public const int EXTERNAL = 278;
  public const int FALSE = 279;
  public const int FAR = 280;
  public const int FORK = 281;
  public const int GUARD = 282;
  public const int IF = 283;
  public const int IMMUTABLE = 284;
  public const int INOUT = 285;
  public const int INCLUDE = 286;
  public const int INITIAL = 287;
  public const int IS = 288;
  public const int ITER = 289;
  public const int LOCK = 290;
  public const int LOOP = 291;
  public const int NEAR = 292;
  public const int NEW = 293;
  public const int ONCE = 294;
  public const int OR = 295;
  public const int OUT = 296;
  public const int PAR = 297;
  public const int PARLOOP = 298;
  public const int POST = 299;
  public const int PRE = 300;
  public const int PRIVATE = 301;
  public const int PROTECT = 302;
  public const int QUIT = 303;
  public const int RAISE = 304;
  public const int READONLY = 305;
  public const int RESULT = 306;
  public const int RETURN = 307;
  public const int ROUT = 308;
  public const int F_ROUT = 309;
  public const int SAME = 310;
  public const int SELF = 311;
  public const int SHARED = 312;
  public const int SYNC = 313;
  public const int THEN = 314;
  public const int TRUE = 315;
  public const int TYPECASE = 316;
  public const int UNLOCK = 317;
  public const int UNTIL_BANG = 318;
  public const int VOID = 319;
  public const int WHEN = 320;
  public const int WHILE_BANG = 321;
  public const int WITH = 322;
  public const int YIELD = 323;
  public const int LPAREN = 324;
  public const int RPAREN = 325;
  public const int LBRACKET = 326;
  public const int RBRACKET = 327;
  public const int LBRACE = 328;
  public const int RBRACE = 329;
  public const int COMMA = 330;
  public const int DOT = 331;
  public const int SEMI = 332;
  public const int COLON = 333;
  public const int UNDER = 334;
  public const int PLUS = 335;
  public const int MINUS = 336;
  public const int TIMES = 337;
  public const int QUOTIENT = 338;
  public const int IS_LT = 339;
  public const int IS_GT = 340;
  public const int SHARP = 341;
  public const int BANG = 342;
  public const int ITER_BANG = 343;
  public const int POW = 344;
  public const int MOD = 345;
  public const int VBAR = 346;
  public const int IS_NEQ = 347;
  public const int IS_LEQ = 349;
  public const int IS_GEQ = 351;
  public const int ASSIGN = 353;
  public const int TRANSFORM = 355;
  public const int IS_EQ = 357;
  public const int NOT = 358;
  public const int DCOLON = 359;
  public const int CHAR_LITERAL = 361;
  public const int STR_LITERAL = 362;
  public const int INT_LITERAL = 363;
  public const int FLOAT_LITERAL = 364;
  public const int IDENTIFIER = 365;
  public const int ABSTRACT_CLASS_NAME = 366;
  public const int yyErrorCode = 256;
 }
 namespace yyParser {
  using System;
  /** thrown for irrecoverable syntax errors and stack overflow.
    */
  public class yyException : System.Exception {
    public yyException (string message) : base (message) {
    }
  }

  /** must be implemented by a scanner object to supply input to the parser.
    */
  public interface yyInput {
    /** move on to next token.
        @return false if positioned beyond tokens.
        @throws IOException on input error.
      */
    bool advance (); // throws java.io.IOException;
    /** classifies current token.
        Should not be called if advance() returned false.
        @return current %token or single character.
      */
    int token ();
    /** associated with current token.
        Should not be called if advance() returned false.
        @return value for token().
      */
    Object value ();
  }
 }
} // close outermost namespace, that MUST HAVE BEEN opened in the prolog
